@@ -1,3 +1,4 @@
+from . import qops as q
 from .archives import *
 from .builtins import *
 from .base_savable import *
@@ -14,13 +15,12 @@ from .records import *
 from .refs import *
 from .types import *
 from .version import *
-from . import analysis
+from . import builtins
 from . import common_helpers
 from . import mongo
-from . import testing
 from . import utils
 
-_ADDITIONAL = ('analysis', 'mongo', 'buitins', 'common_helpers', 'testing', 'utils')
+_ADDITIONAL = ('analysis', 'mongo', 'builtins', 'common_helpers', 'utils', 'q')
 
 __all__ = (archives.__all__ + comparators.__all__ + depositors.__all__ + exceptions.__all__ +
            historians.__all__ + convenience.__all__ + process.__all__ + types.__all__ +
