@@ -2,14 +2,17 @@ import pymongo
 import pymongo.uri_parser
 
 from . import historians
-from . import mongo
 
-__all__ = 'archive', 'historian'
+__all__ = 'create_archive', 'create_historian'
+
+from . import plugins
 
 
-def archive(uri: str):
+def create_archive(uri: str):
     """Create an archive type based on a uri string"""
     if uri.startswith('mongodb'):
+        from . import mongo  # pylint: disable=import-outside-toplevel
+
         # Format is:
         # mongodb://[username:password@]host1[:port1][,...hostN[:portN]][/[database][?options]]
         parsed = pymongo.uri_parser.parse_uri(uri)
@@ -22,6 +25,10 @@ def archive(uri: str):
     raise ValueError("Unknown archive string: {}".format(uri))
 
 
-def historian(archive_uri: str):
+def create_historian(archive_uri: str, apply_plugins=True):
     """Convenience function to create a standard historian directly from an archive URI"""
-    return historians.Historian(archive(archive_uri))
+    historian = historians.Historian(create_archive(archive_uri))
+    if apply_plugins:
+        historian.register_types(plugins.get_types())
+
+    return historian
