@@ -1,4 +1,4 @@
-__all__ = ('NotFound', 'ModificationError', 'ObjectDeleted')
+__all__ = 'NotFound', 'ModificationError', 'ObjectDeleted', 'DuplicateKeyError'
 
 
 class NotFound(Exception):
@@ -9,5 +9,9 @@ class ModificationError(Exception):
     """Raised when a modification of the history encountered a problem"""
 
 
-class ObjectDeleted(Exception):
+class ObjectDeleted(NotFound):
     """Raise when the user tries to interact with a deleted object"""
+
+
+class DuplicateKeyError(Exception):
+    """Indicates that a uniqueness constraint was violated"""
