@@ -1,5 +1,4 @@
 """References module"""
-import uuid
 
 from . import exceptions
 from . import types
@@ -10,26 +9,34 @@ __all__ = ('ObjRef',)
 
 class ObjRef(types.SavableObject):
     """A reference to an object instance"""
-    TYPE_ID = uuid.UUID('633c7035-64fe-4d87-a91e-3b7abd8a6a28')
+    TYPE_ID = records.SnapshotRef.TYPE_ID
     IMMUTABLE = True
 
     _obj = None
     _ref = None
     _loader = None
 
-    def __init__(self, obj=None, historian=None):
-        super().__init__(historian)
-        assert not (obj is not None and self._historian.is_primitive(obj)), \
+    def __init__(self, obj=None):
+        super().__init__()
+        assert not (obj is not None and types.is_primitive(obj)), \
             "Can't create a reference to a primitive type"
         self._obj = obj
 
+    def __bool__(self):
+        """Test if this is a null reference"""
+        return self._obj is not None or self._ref is not None
+
     def __str__(self):
-        desc = "ref: "
+        desc = ["ObjRef('"]
         if self._obj is not None:
-            desc += str(self._obj)
+            desc.append(str(self._obj))
         else:
-            desc += str(self._ref)
-        return desc
+            desc.append(str(self._ref))
+        desc.append("')")
+        return "".join(desc)
+
+    def __repr__(self):
+        return "ObjRef({})".format(self._obj if self._obj is not None else self._ref)
 
     def __call__(self, update=False):
         """Get the object being referenced.  If update is called then the latest version
@@ -74,7 +81,7 @@ class ObjRef(types.SavableObject):
             ref = self._ref
 
         if ref is not None:
-            return ref.to_list()
+            return ref.to_dict()
 
         return None
 
@@ -82,7 +89,12 @@ class ObjRef(types.SavableObject):
         super(ObjRef, self).load_instance_state(saved_state, loader)
         # Rely on class default values for members
         if saved_state is not None:
-            self._ref = records.Ref(*saved_state)
+            if isinstance(saved_state, list):
+                # Legacy version
+                self._ref = records.SnapshotRef(*saved_state)
+            else:
+                # New version is dict
+                self._ref = records.SnapshotRef(**saved_state)
             self._loader = loader
 
 
