@@ -1,6 +1,5 @@
 from abc import ABCMeta, abstractmethod
 import datetime
-import typing
 import uuid
 
 try:  # Python3
@@ -8,7 +7,8 @@ try:  # Python3
 except ImportError:  # Python < 3.6
     from pyblake2 import blake2b
 
-from . import exceptions
+import mincepy
+from . import depositors
 
 __all__ = 'Savable', 'Comparable', 'Object', 'SavableObject', 'PRIMITIVE_TYPES'
 
@@ -17,6 +17,10 @@ PRIMITIVE_TYPES = (bool, int, float, str, dict, list, type(None), bytes, uuid.UU
                    datetime.datetime)
 
 
+def is_primitive(obj):
+    return isinstance(obj, PRIMITIVE_TYPES)
+
+
 class Savable(metaclass=ABCMeta):
     """Interface for an object that can save an load its instance state"""
     TYPE_ID = None
@@ -25,11 +29,11 @@ class Savable(metaclass=ABCMeta):
         assert self.TYPE_ID is not None, "Must set the TYPE_ID for an object to be savable"
 
     @abstractmethod
-    def save_instance_state(self, saver):
+    def save_instance_state(self, saver: depositors.Saver):
         """Save the instance state of an object, should return a saved instance"""
 
     @abstractmethod
-    def load_instance_state(self, saved_state, loader):
+    def load_instance_state(self, saved_state, loader: depositors.Loader):
         """Take the given object and load the instance state into it"""
 
 
@@ -52,51 +56,11 @@ class Object(Comparable, metaclass=ABCMeta):
 class SavableObject(Object, Savable, metaclass=ABCMeta):
     """A class that is both savable and comparable"""
 
-    def __init__(self, historian=None):
+    _historian = None
+
+    def __init__(self):
         super().__init__()
-        from . import history
-        # Tell the historian that we've been created
-        self._historian = historian or history.get_historian()
-        assert self._historian is not None, \
-            "Must provide a valid historian or set one globally using mincepy.set_historian()"
-
-        if self._historian is not None:
-            self._historian.created(self)
-
-    @property
-    def obj_id(self):
-        try:
-            return self._historian.get_obj_id(self)
-        except exceptions.NotFound:
-            return None
-
-    def get_meta(self) -> dict:
-        """Get the metadata dictionary for this object"""
-        return self._historian.get_meta(self)
-
-    def set_meta(self, meta: typing.Optional[dict]):
-        """Set the metadata dictionary for this object"""
-        self._historian.set_meta(self, meta)
-
-    def update_meta(self, meta: dict):
-        """Update the metadata dictionary for this object"""
-        self._historian.update_meta(self, meta)
-
-    def is_saved(self) -> bool:
-        return self.obj_id is not None
-
-    def save(self, with_meta=None, return_sref=False):
-        """Save the object"""
-        return self._historian.save(self, with_meta=with_meta, return_sref=return_sref)
-
-    def sync(self):
-        """Update the state of this object by loading the latest version from the historian"""
-        return self._historian.sync(self)
-
-    def load_instance_state(self, saved_state, loader):
-        """Take the given object and load the instance state into it"""
-        super().load_instance_state(saved_state, loader)
-        self._historian = loader.get_historian()
+        mincepy.process.CreatorsRegistry.created(self)
 
 
 class Equator:
@@ -150,7 +114,7 @@ class Equator:
         return self._hasher(*self.yield_hashables(obj))
 
     def eq(self, obj1, obj2) -> bool:  # pylint: disable=invalid-name
-        if not type(obj1) == type(obj2):
+        if not type(obj1) == type(obj2):  # pylint: disable=unidiomatic-typecheck
             return False
 
         try:
